(set-logic BV)

(declare-const inv_s (_ BitVec 1))
(declare-const inv_s_ctz_s (_ BitVec 1))
(declare-const t (_ BitVec 1))
(declare-const xlo (_ BitVec 1))
(declare-const xhi (_ BitVec 1))

(define-fun mcb ((xlo (_ BitVec 1)) (xhi (_ BitVec 1)) (v (_ BitVec 1))) Bool
 (and
  (= (bvand xhi v) v)
  (= (bvor xlo v) v)
 )
)

(define-fun ctz ((v (_ BitVec 1))) (_ BitVec 1)
 (ite (= ((_ extract 0 0) v) #b0) (_ bv1 1) (_ bv0 1))
)

(define-fun clz ((v (_ BitVec 1))) (_ BitVec 1)
 (ite (= ((_ extract 0 0) v) #b0) (_ bv1 1) (_ bv0 1))
)

(define-fun cto ((v (_ BitVec 1))) (_ BitVec 1)
 (ctz (bvnot v))
)

(define-fun clo ((v (_ BitVec 1))) (_ BitVec 1)
 (clz (bvnot v))
)

(define-fun odd ((v (_ BitVec 1))) Bool
 (= ((_ extract 0 0) v) #b1)
)

(define-fun zero () (_ BitVec 1)
 (_ bv0 1)
)

(define-fun one () (_ BitVec 1)
 (_ bv1 1)
)

(define-fun ones () (_ BitVec 1)
 (bvnot zero)
)

(define-fun w () (_ BitVec 1)
 (_ bv1 1)
)

(define-fun addo ((a (_ BitVec 1)) (b (_ BitVec 1))) Bool
 (let ((zext_a ((_ zero_extend 1) a)) (zext_b ((_ zero_extend 1) b)))
  (= ((_ extract 1 1) (bvadd zext_a zext_b)) #b1)
 )
)

(define-fun mulo ((a (_ BitVec 1)) (b (_ BitVec 1))) Bool
 (let ((zext_a ((_ zero_extend 1) a)) (zext_b ((_ zero_extend 1) b)))
  (distinct
   ((_ extract 0 0) (bvlshr (bvmul zext_a zext_b) ((_ zero_extend 1) (_ bv1 1))))
   zero
  )
 )
)

(define-fun CC ((t (_ BitVec 1)) (xlo (_ BitVec 1)) (xhi (_ BitVec 1))) Bool
 (and
  (=>
   (= t ones)
   (or
     (mcb xlo xhi zero)
     (mcb xlo xhi one)
   )
  )
  (=>
   (distinct t ones)
   (and
    (not (mulo xlo t))
    (exists ((y (_ BitVec 1)))
     (and
      (mcb xlo xhi y)
      (bvugt y zero)
      (not (mulo y t))
     )
    )
   )
  )
 )
)

(define-fun LIT ((x (_ BitVec 1)) (s (_ BitVec 1))) Bool
 (= (bvudiv s x) t)
)

; valid xlo/xhi pairs
(assert (= (bvor (bvnot xlo) xhi) ones))

(assert
 (distinct
  (CC t xlo xhi)
  (exists ((x (_ BitVec 1)) (s (_ BitVec 1)))
   (and
    (mcb xlo xhi x)
    (LIT x s)
   )
  )
 )
)

(check-sat)
(exit)

