(set-logic BV)

(declare-const inv_s (_ BitVec 8))
(declare-const inv_s_ctz_s (_ BitVec 8))
(declare-const t (_ BitVec 8))
(declare-const xlo (_ BitVec 8))
(declare-const xhi (_ BitVec 8))

(define-fun mcb ((xlo (_ BitVec 8)) (xhi (_ BitVec 8)) (v (_ BitVec 8))) Bool
 (and
  (= (bvand xhi v) v)
  (= (bvor xlo v) v)
 )
)

(define-fun ctz ((v (_ BitVec 8))) (_ BitVec 8)
 (ite (= ((_ extract 0 0) v) #b0) (ite (= ((_ extract 1 1) v) #b0) (ite (= ((_ extract 2 2) v) #b0) (ite (= ((_ extract 3 3) v) #b0) (ite (= ((_ extract 4 4) v) #b0) (ite (= ((_ extract 5 5) v) #b0) (ite (= ((_ extract 6 6) v) #b0) (ite (= ((_ extract 7 7) v) #b0) (_ bv8 8) (_ bv7 8)) (_ bv6 8)) (_ bv5 8)) (_ bv4 8)) (_ bv3 8)) (_ bv2 8)) (_ bv1 8)) (_ bv0 8))
)

(define-fun clz ((v (_ BitVec 8))) (_ BitVec 8)
 (ite (= ((_ extract 7 7) v) #b0) (ite (= ((_ extract 6 6) v) #b0) (ite (= ((_ extract 5 5) v) #b0) (ite (= ((_ extract 4 4) v) #b0) (ite (= ((_ extract 3 3) v) #b0) (ite (= ((_ extract 2 2) v) #b0) (ite (= ((_ extract 1 1) v) #b0) (ite (= ((_ extract 0 0) v) #b0) (_ bv8 8) (_ bv7 8)) (_ bv6 8)) (_ bv5 8)) (_ bv4 8)) (_ bv3 8)) (_ bv2 8)) (_ bv1 8)) (_ bv0 8))
)

(define-fun cto ((v (_ BitVec 8))) (_ BitVec 8)
 (ctz (bvnot v))
)

(define-fun clo ((v (_ BitVec 8))) (_ BitVec 8)
 (clz (bvnot v))
)

(define-fun odd ((v (_ BitVec 8))) Bool
 (= ((_ extract 0 0) v) #b1)
)

(define-fun zero () (_ BitVec 8)
 (_ bv0 8)
)

(define-fun one () (_ BitVec 8)
 (_ bv1 8)
)

(define-fun ones () (_ BitVec 8)
 (bvnot zero)
)

(define-fun w () (_ BitVec 8)
 (_ bv8 8)
)

(define-fun addo ((a (_ BitVec 8)) (b (_ BitVec 8))) Bool
 (let ((zext_a ((_ zero_extend 1) a)) (zext_b ((_ zero_extend 1) b)))
  (= ((_ extract 8 8) (bvadd zext_a zext_b)) #b1)
 )
)

(define-fun mulo ((a (_ BitVec 8)) (b (_ BitVec 8))) Bool
 (let ((zext_a ((_ zero_extend 8) a)) (zext_b ((_ zero_extend 8) b)))
  (distinct
   ((_ extract 7 0) (bvlshr (bvmul zext_a zext_b) ((_ zero_extend 8) (_ bv8 8))))
   zero
  )
 )
)

(define-fun CC ((t (_ BitVec 8)) (xlo (_ BitVec 8)) (xhi (_ BitVec 8))) Bool
 (and
  (=>
   (= t ones)
   (or
     (mcb xlo xhi zero)
     (mcb xlo xhi one)
   )
  )
  (=>
   (distinct t ones)
   (and
    (not (mulo xlo t))
    (exists ((y (_ BitVec 8)))
     (and
      (mcb xlo xhi y)
      (bvugt y zero)
      (not (mulo y t))
     )
    )
   )
  )
 )
)

(define-fun LIT ((x (_ BitVec 8)) (s (_ BitVec 8))) Bool
 (= (bvudiv s x) t)
)

; valid xlo/xhi pairs
(assert (= (bvor (bvnot xlo) xhi) ones))

(assert
 (distinct
  (CC t xlo xhi)
  (exists ((x (_ BitVec 8)) (s (_ BitVec 8)))
   (and
    (mcb xlo xhi x)
    (LIT x s)
   )
  )
 )
)

(check-sat)
(exit)

