(set-logic BV)

(declare-const s (_ BitVec 2))
(declare-const t (_ BitVec 2))
(declare-const xlo (_ BitVec 2))
(declare-const xhi (_ BitVec 2))

(define-fun mcb ((xlo (_ BitVec 2)) (xhi (_ BitVec 2)) (v (_ BitVec 2))) Bool
 (and
  (= (bvand xhi v) v)
  (= (bvor xlo v) v)
 )
)

(define-fun ctz ((v (_ BitVec 2))) (_ BitVec 2)
 (ite (= ((_ extract 0 0) v) #b0) (ite (= ((_ extract 1 1) v) #b0) (_ bv2 2) (_ bv1 2)) (_ bv0 2))
)

(define-fun clz ((v (_ BitVec 2))) (_ BitVec 2)
 (ite (= ((_ extract 1 1) v) #b0) (ite (= ((_ extract 0 0) v) #b0) (_ bv2 2) (_ bv1 2)) (_ bv0 2))
)

(define-fun cto ((v (_ BitVec 2))) (_ BitVec 2)
 (ctz (bvnot v))
)

(define-fun clo ((v (_ BitVec 2))) (_ BitVec 2)
 (clz (bvnot v))
)

(define-fun odd ((v (_ BitVec 2))) Bool
 (= ((_ extract 0 0) v) #b1)
)

(define-fun zero () (_ BitVec 2)
 (_ bv0 2)
)

(define-fun one () (_ BitVec 2)
 (_ bv1 2)
)

(define-fun ones () (_ BitVec 2)
 (bvnot zero)
)

(define-fun w () (_ BitVec 2)
 (_ bv2 2)
)

(define-fun IC ((s (_ BitVec 2)) (t (_ BitVec 2)) (xlo (_ BitVec 2)) (xhi (_ BitVec 2))) Bool
 (and
  (= (bvand (bvor (bvneg s) s) t) t)
  (or
   (= s zero)
   (and
    (=>
     (odd s)
     (exists ((inv_s (_ BitVec 2)))
      (and
       ; modular inverse of s
       (= (bvmul s inv_s) one)
       (mcb xlo xhi (bvmul t inv_s))
      )
     )
    )
    (=>
     (not (odd s))
     (exists ((inv_s_ctz_s (_ BitVec 2)))
      (and
       ; modular inverse of (bvlshr s (ctz s))
       (= (bvmul (bvlshr s (ctz s)) inv_s_ctz_s) one)
       (mcb
        (bvshl xlo (ctz s))
        (bvshl xhi (ctz s))
        (bvshl (bvmul (bvlshr t (ctz s)) inv_s_ctz_s) (ctz s)))
      )
     )
    )
   )
  )
 )
)

(define-fun LIT ((x (_ BitVec 2))) Bool
 (= (bvmul x s) t)
)

; valid xlo/xhi pairs
(assert (= (bvor (bvnot xlo) xhi) ones))

(assert
 (distinct
  (IC s t xlo xhi)
  (exists ((x (_ BitVec 2)))
   (and
    (mcb xlo xhi x)
    (LIT x)
   )
  )
 )
)

(check-sat)
(exit)

