(set-logic BV)

(declare-const s (_ BitVec 3))
(declare-const t (_ BitVec 3))
(declare-const xlo (_ BitVec 3))
(declare-const xhi (_ BitVec 3))

(define-fun mcb ((xlo (_ BitVec 3)) (xhi (_ BitVec 3)) (v (_ BitVec 3))) Bool
 (and
  (= (bvand xhi v) v)
  (= (bvor xlo v) v)
 )
)

(define-fun ctz ((v (_ BitVec 3))) (_ BitVec 3)
 (ite (= ((_ extract 0 0) v) #b0) (ite (= ((_ extract 1 1) v) #b0) (ite (= ((_ extract 2 2) v) #b0) (_ bv3 3) (_ bv2 3)) (_ bv1 3)) (_ bv0 3))
)

(define-fun clz ((v (_ BitVec 3))) (_ BitVec 3)
 (ite (= ((_ extract 2 2) v) #b0) (ite (= ((_ extract 1 1) v) #b0) (ite (= ((_ extract 0 0) v) #b0) (_ bv3 3) (_ bv2 3)) (_ bv1 3)) (_ bv0 3))
)

(define-fun cto ((v (_ BitVec 3))) (_ BitVec 3)
 (ctz (bvnot v))
)

(define-fun clo ((v (_ BitVec 3))) (_ BitVec 3)
 (clz (bvnot v))
)

(define-fun odd ((v (_ BitVec 3))) Bool
 (= ((_ extract 0 0) v) #b1)
)

(define-fun zero () (_ BitVec 3)
 (_ bv0 3)
)

(define-fun one () (_ BitVec 3)
 (_ bv1 3)
)

(define-fun ones () (_ BitVec 3)
 (bvnot zero)
)

(define-fun w () (_ BitVec 3)
 (_ bv3 3)
)

(define-fun IC ((s (_ BitVec 3)) (t (_ BitVec 3)) (xlo (_ BitVec 3)) (xhi (_ BitVec 3))) Bool
 (and
  (= (bvand (bvor (bvneg s) s) t) t)
  (or
   (= s zero)
   (and
    (=>
     (odd s)
     (exists ((inv_s (_ BitVec 3)))
      (and
       ; modular inverse of s
       (= (bvmul s inv_s) one)
       (mcb xlo xhi (bvmul t inv_s))
      )
     )
    )
    (=>
     (not (odd s))
     (exists ((inv_s_ctz_s (_ BitVec 3)))
      (and
       ; modular inverse of (bvlshr s (ctz s))
       (= (bvmul (bvlshr s (ctz s)) inv_s_ctz_s) one)
       (mcb
        (bvshl xlo (ctz s))
        (bvshl xhi (ctz s))
        (bvshl (bvmul (bvlshr t (ctz s)) inv_s_ctz_s) (ctz s)))
      )
     )
    )
   )
  )
 )
)

(define-fun LIT ((x (_ BitVec 3))) Bool
 (= (bvmul x s) t)
)

; valid xlo/xhi pairs
(assert (= (bvor (bvnot xlo) xhi) ones))

(assert
 (distinct
  (IC s t xlo xhi)
  (exists ((x (_ BitVec 3)))
   (and
    (mcb xlo xhi x)
    (LIT x)
   )
  )
 )
)

(check-sat)
(exit)

