(set-logic BV)

(declare-const s (_ BitVec 8))
(declare-const t (_ BitVec 8))
(declare-const xlo (_ BitVec 8))
(declare-const xhi (_ BitVec 8))

(define-fun mcb ((xlo (_ BitVec 8)) (xhi (_ BitVec 8)) (v (_ BitVec 8))) Bool
 (and
  (= (bvand xhi v) v)
  (= (bvor xlo v) v)
 )
)

(define-fun ctz ((v (_ BitVec 8))) (_ BitVec 8)
 (ite (= ((_ extract 0 0) v) #b0) (ite (= ((_ extract 1 1) v) #b0) (ite (= ((_ extract 2 2) v) #b0) (ite (= ((_ extract 3 3) v) #b0) (ite (= ((_ extract 4 4) v) #b0) (ite (= ((_ extract 5 5) v) #b0) (ite (= ((_ extract 6 6) v) #b0) (ite (= ((_ extract 7 7) v) #b0) (_ bv8 8) (_ bv7 8)) (_ bv6 8)) (_ bv5 8)) (_ bv4 8)) (_ bv3 8)) (_ bv2 8)) (_ bv1 8)) (_ bv0 8))
)

(define-fun clz ((v (_ BitVec 8))) (_ BitVec 8)
 (ite (= ((_ extract 7 7) v) #b0) (ite (= ((_ extract 6 6) v) #b0) (ite (= ((_ extract 5 5) v) #b0) (ite (= ((_ extract 4 4) v) #b0) (ite (= ((_ extract 3 3) v) #b0) (ite (= ((_ extract 2 2) v) #b0) (ite (= ((_ extract 1 1) v) #b0) (ite (= ((_ extract 0 0) v) #b0) (_ bv8 8) (_ bv7 8)) (_ bv6 8)) (_ bv5 8)) (_ bv4 8)) (_ bv3 8)) (_ bv2 8)) (_ bv1 8)) (_ bv0 8))
)

(define-fun cto ((v (_ BitVec 8))) (_ BitVec 8)
 (ctz (bvnot v))
)

(define-fun clo ((v (_ BitVec 8))) (_ BitVec 8)
 (clz (bvnot v))
)

(define-fun odd ((v (_ BitVec 8))) Bool
 (= ((_ extract 0 0) v) #b1)
)

(define-fun zero () (_ BitVec 8)
 (_ bv0 8)
)

(define-fun one () (_ BitVec 8)
 (_ bv1 8)
)

(define-fun ones () (_ BitVec 8)
 (bvnot zero)
)

(define-fun w () (_ BitVec 8)
 (_ bv8 8)
)

(define-fun IC ((s (_ BitVec 8)) (t (_ BitVec 8)) (xlo (_ BitVec 8)) (xhi (_ BitVec 8))) Bool
 (and
  (= (bvand (bvor (bvneg s) s) t) t)
  (or
   (= s zero)
   (and
    (=>
     (odd s)
     (exists ((inv_s (_ BitVec 8)))
      (and
       ; modular inverse of s
       (= (bvmul s inv_s) one)
       (mcb xlo xhi (bvmul t inv_s))
      )
     )
    )
    (=>
     (not (odd s))
     (exists ((inv_s_ctz_s (_ BitVec 8)))
      (and
       ; modular inverse of (bvlshr s (ctz s))
       (= (bvmul (bvlshr s (ctz s)) inv_s_ctz_s) one)
       (mcb
        (bvshl xlo (ctz s))
        (bvshl xhi (ctz s))
        (bvshl (bvmul (bvlshr t (ctz s)) inv_s_ctz_s) (ctz s)))
      )
     )
    )
   )
  )
 )
)

(define-fun LIT ((x (_ BitVec 8))) Bool
 (= (bvmul x s) t)
)

; valid xlo/xhi pairs
(assert (= (bvor (bvnot xlo) xhi) ones))

(assert
 (distinct
  (IC s t xlo xhi)
  (exists ((x (_ BitVec 8)))
   (and
    (mcb xlo xhi x)
    (LIT x)
   )
  )
 )
)

(check-sat)
(exit)

