(set-logic BV)

(declare-const s (_ BitVec 7))
(declare-const t (_ BitVec 7))
(declare-const xlo (_ BitVec 7))
(declare-const xhi (_ BitVec 7))

(define-fun mcb ((xlo (_ BitVec 7)) (xhi (_ BitVec 7)) (v (_ BitVec 7))) Bool
 (and
  (= (bvand xhi v) v)
  (= (bvor xlo v) v)
 )
)

(define-fun ctz ((v (_ BitVec 7))) (_ BitVec 7)
 (ite (= ((_ extract 0 0) v) #b0) (ite (= ((_ extract 1 1) v) #b0) (ite (= ((_ extract 2 2) v) #b0) (ite (= ((_ extract 3 3) v) #b0) (ite (= ((_ extract 4 4) v) #b0) (ite (= ((_ extract 5 5) v) #b0) (ite (= ((_ extract 6 6) v) #b0) (_ bv7 7) (_ bv6 7)) (_ bv5 7)) (_ bv4 7)) (_ bv3 7)) (_ bv2 7)) (_ bv1 7)) (_ bv0 7))
)

(define-fun clz ((v (_ BitVec 7))) (_ BitVec 7)
 (ite (= ((_ extract 6 6) v) #b0) (ite (= ((_ extract 5 5) v) #b0) (ite (= ((_ extract 4 4) v) #b0) (ite (= ((_ extract 3 3) v) #b0) (ite (= ((_ extract 2 2) v) #b0) (ite (= ((_ extract 1 1) v) #b0) (ite (= ((_ extract 0 0) v) #b0) (_ bv7 7) (_ bv6 7)) (_ bv5 7)) (_ bv4 7)) (_ bv3 7)) (_ bv2 7)) (_ bv1 7)) (_ bv0 7))
)

(define-fun cto ((v (_ BitVec 7))) (_ BitVec 7)
 (ctz (bvnot v))
)

(define-fun clo ((v (_ BitVec 7))) (_ BitVec 7)
 (clz (bvnot v))
)

(define-fun odd ((v (_ BitVec 7))) Bool
 (= ((_ extract 0 0) v) #b1)
)

(define-fun zero () (_ BitVec 7)
 (_ bv0 7)
)

(define-fun one () (_ BitVec 7)
 (_ bv1 7)
)

(define-fun ones () (_ BitVec 7)
 (bvnot zero)
)

(define-fun w () (_ BitVec 7)
 (_ bv7 7)
)

(define-fun addo ((a (_ BitVec 7)) (b (_ BitVec 7))) Bool
 (let ((zext_a ((_ zero_extend 1) a)) (zext_b ((_ zero_extend 1) b)))
  (= ((_ extract 7 7) (bvadd zext_a zext_b)) #b1)
 )
)

(define-fun mulo ((a (_ BitVec 7)) (b (_ BitVec 7))) Bool
 (let ((zext_a ((_ zero_extend 7) a)) (zext_b ((_ zero_extend 7) b)))
  (distinct
   ((_ extract 6 0) (bvlshr (bvmul zext_a zext_b) ((_ zero_extend 7) (_ bv7 7))))
   zero
  )
 )
)

(define-fun IC ((s (_ BitVec 7)) (t (_ BitVec 7)) (xlo (_ BitVec 7)) (xhi (_ BitVec 7))) Bool
 (and
  (bvuge (bvand (bvsub (bvadd t t) s) s) t)
  (=>
   (= s t)
   (or (= xlo zero) (bvugt xhi t))
  )
  (=>
   (distinct s t)
   (exists ((y (_ BitVec 7)))
    (and
     (mcb xlo xhi y)
     (bvugt y t)
     (= (bvurem (bvsub s t) y) zero)
    )
   )
  )
 )
)

(define-fun LIT ((x (_ BitVec 7))) Bool
 (= (bvurem s x) t)
)

; valid xlo/xhi pairs
(assert (= (bvor (bvnot xlo) xhi) ones))

(assert
 (distinct
  (IC s t xlo xhi)
  (exists ((x (_ BitVec 7)))
   (and
    (mcb xlo xhi x)
    (LIT x)
   )
  )
 )
)

(check-sat)
(exit)

