(set-logic BV)

(declare-const s (_ BitVec 1))
(declare-const t (_ BitVec 1))
(declare-const xlo (_ BitVec 1))
(declare-const xhi (_ BitVec 1))

(define-fun mcb ((xlo (_ BitVec 1)) (xhi (_ BitVec 1)) (v (_ BitVec 1))) Bool
 (and
  (= (bvand xhi v) v)
  (= (bvor xlo v) v)
 )
)

(define-fun ctz ((v (_ BitVec 1))) (_ BitVec 1)
 (ite (= ((_ extract 0 0) v) #b0) (_ bv1 1) (_ bv0 1))
)

(define-fun clz ((v (_ BitVec 1))) (_ BitVec 1)
 (ite (= ((_ extract 0 0) v) #b0) (_ bv1 1) (_ bv0 1))
)

(define-fun cto ((v (_ BitVec 1))) (_ BitVec 1)
 (ctz (bvnot v))
)

(define-fun clo ((v (_ BitVec 1))) (_ BitVec 1)
 (clz (bvnot v))
)

(define-fun odd ((v (_ BitVec 1))) Bool
 (= ((_ extract 0 0) v) #b1)
)

(define-fun zero () (_ BitVec 1)
 (_ bv0 1)
)

(define-fun one () (_ BitVec 1)
 (_ bv1 1)
)

(define-fun ones () (_ BitVec 1)
 (bvnot zero)
)

(define-fun w () (_ BitVec 1)
 (_ bv1 1)
)

(define-fun addo ((a (_ BitVec 1)) (b (_ BitVec 1))) Bool
 (let ((zext_a ((_ zero_extend 1) a)) (zext_b ((_ zero_extend 1) b)))
  (= ((_ extract 1 1) (bvadd zext_a zext_b)) #b1)
 )
)

(define-fun mulo ((a (_ BitVec 1)) (b (_ BitVec 1))) Bool
 (let ((zext_a ((_ zero_extend 1) a)) (zext_b ((_ zero_extend 1) b)))
  (distinct
   ((_ extract 0 0) (bvlshr (bvmul zext_a zext_b) ((_ zero_extend 1) (_ bv1 1))))
   zero
  )
 )
)

(define-fun IC ((s (_ BitVec 1)) (t (_ BitVec 1)) (xlo (_ BitVec 1)) (xhi (_ BitVec 1))) Bool
 (and
  (bvuge (bvnot (bvneg s)) t)
  (=>
   (or (= s zero) (= t ones))
   (mcb xlo xhi t)
  )
  (=>
   (and
    (distinct s zero)
    (distinct t ones)
   )
   (exists ((y (_ BitVec 1)))
    (and
     (mcb xlo xhi (bvadd (bvmul s y) t))
     (not (mulo s y))
     (not (addo (bvmul s y) t))
    )
   )
  )
 )
)

(define-fun LIT ((x (_ BitVec 1))) Bool
 (= (bvurem x s) t)
)

; valid xlo/xhi pairs
(assert (= (bvor (bvnot xlo) xhi) ones))

(assert
 (distinct
  (IC s t xlo xhi)
  (exists ((x (_ BitVec 1)))
   (and
    (mcb xlo xhi x)
    (LIT x)
   )
  )
 )
)

(check-sat)
(exit)

