(set-logic BV)

(declare-const s (_ BitVec 2))
(declare-const t (_ BitVec 2))
(declare-const xlo (_ BitVec 2))
(declare-const xhi (_ BitVec 2))

(define-fun mcb ((xlo (_ BitVec 2)) (xhi (_ BitVec 2)) (v (_ BitVec 2))) Bool
 (and
  (= (bvand xhi v) v)
  (= (bvor xlo v) v)
 )
)

(define-fun ctz ((v (_ BitVec 2))) (_ BitVec 2)
 (ite (= ((_ extract 0 0) v) #b0) (ite (= ((_ extract 1 1) v) #b0) (_ bv2 2) (_ bv1 2)) (_ bv0 2))
)

(define-fun clz ((v (_ BitVec 2))) (_ BitVec 2)
 (ite (= ((_ extract 1 1) v) #b0) (ite (= ((_ extract 0 0) v) #b0) (_ bv2 2) (_ bv1 2)) (_ bv0 2))
)

(define-fun cto ((v (_ BitVec 2))) (_ BitVec 2)
 (ctz (bvnot v))
)

(define-fun clo ((v (_ BitVec 2))) (_ BitVec 2)
 (clz (bvnot v))
)

(define-fun odd ((v (_ BitVec 2))) Bool
 (= ((_ extract 0 0) v) #b1)
)

(define-fun zero () (_ BitVec 2)
 (_ bv0 2)
)

(define-fun one () (_ BitVec 2)
 (_ bv1 2)
)

(define-fun ones () (_ BitVec 2)
 (bvnot zero)
)

(define-fun w () (_ BitVec 2)
 (_ bv2 2)
)

(define-fun addo ((a (_ BitVec 2)) (b (_ BitVec 2))) Bool
 (let ((zext_a ((_ zero_extend 1) a)) (zext_b ((_ zero_extend 1) b)))
  (= ((_ extract 2 2) (bvadd zext_a zext_b)) #b1)
 )
)

(define-fun mulo ((a (_ BitVec 2)) (b (_ BitVec 2))) Bool
 (let ((zext_a ((_ zero_extend 2) a)) (zext_b ((_ zero_extend 2) b)))
  (distinct
   ((_ extract 1 0) (bvlshr (bvmul zext_a zext_b) ((_ zero_extend 2) (_ bv2 2))))
   zero
  )
 )
)

(define-fun IC ((s (_ BitVec 2)) (t (_ BitVec 2)) (xlo (_ BitVec 2)) (xhi (_ BitVec 2))) Bool
 (and
  (bvuge (bvnot (bvneg s)) t)
  (=>
   (or (= s zero) (= t ones))
   (mcb xlo xhi t)
  )
  (=>
   (and
    (distinct s zero)
    (distinct t ones)
   )
   (exists ((y (_ BitVec 2)))
    (and
     (mcb xlo xhi (bvadd (bvmul s y) t))
     (not (mulo s y))
     (not (addo (bvmul s y) t))
    )
   )
  )
 )
)

(define-fun LIT ((x (_ BitVec 2))) Bool
 (= (bvurem x s) t)
)

; valid xlo/xhi pairs
(assert (= (bvor (bvnot xlo) xhi) ones))

(assert
 (distinct
  (IC s t xlo xhi)
  (exists ((x (_ BitVec 2)))
   (and
    (mcb xlo xhi x)
    (LIT x)
   )
  )
 )
)

(check-sat)
(exit)

