(set-logic BV)

(declare-const s (_ BitVec 4))
(declare-const t (_ BitVec 4))
(declare-const xlo (_ BitVec 4))
(declare-const xhi (_ BitVec 4))

(define-fun mcb ((xlo (_ BitVec 4)) (xhi (_ BitVec 4)) (v (_ BitVec 4))) Bool
 (and
  (= (bvand xhi v) v)
  (= (bvor xlo v) v)
 )
)

(define-fun ctz ((v (_ BitVec 4))) (_ BitVec 4)
 (ite (= ((_ extract 0 0) v) #b0) (ite (= ((_ extract 1 1) v) #b0) (ite (= ((_ extract 2 2) v) #b0) (ite (= ((_ extract 3 3) v) #b0) (_ bv4 4) (_ bv3 4)) (_ bv2 4)) (_ bv1 4)) (_ bv0 4))
)

(define-fun clz ((v (_ BitVec 4))) (_ BitVec 4)
 (ite (= ((_ extract 3 3) v) #b0) (ite (= ((_ extract 2 2) v) #b0) (ite (= ((_ extract 1 1) v) #b0) (ite (= ((_ extract 0 0) v) #b0) (_ bv4 4) (_ bv3 4)) (_ bv2 4)) (_ bv1 4)) (_ bv0 4))
)

(define-fun cto ((v (_ BitVec 4))) (_ BitVec 4)
 (ctz (bvnot v))
)

(define-fun clo ((v (_ BitVec 4))) (_ BitVec 4)
 (clz (bvnot v))
)

(define-fun odd ((v (_ BitVec 4))) Bool
 (= ((_ extract 0 0) v) #b1)
)

(define-fun zero () (_ BitVec 4)
 (_ bv0 4)
)

(define-fun one () (_ BitVec 4)
 (_ bv1 4)
)

(define-fun ones () (_ BitVec 4)
 (bvnot zero)
)

(define-fun w () (_ BitVec 4)
 (_ bv4 4)
)

(define-fun addo ((a (_ BitVec 4)) (b (_ BitVec 4))) Bool
 (let ((zext_a ((_ zero_extend 1) a)) (zext_b ((_ zero_extend 1) b)))
  (= ((_ extract 4 4) (bvadd zext_a zext_b)) #b1)
 )
)

(define-fun mulo ((a (_ BitVec 4)) (b (_ BitVec 4))) Bool
 (let ((zext_a ((_ zero_extend 4) a)) (zext_b ((_ zero_extend 4) b)))
  (distinct
   ((_ extract 3 0) (bvlshr (bvmul zext_a zext_b) ((_ zero_extend 4) (_ bv4 4))))
   zero
  )
 )
)

(define-fun IC ((s (_ BitVec 4)) (t (_ BitVec 4)) (xlo (_ BitVec 4)) (xhi (_ BitVec 4))) Bool
 (and
  (bvuge (bvnot (bvneg s)) t)
  (=>
   (or (= s zero) (= t ones))
   (mcb xlo xhi t)
  )
  (=>
   (and
    (distinct s zero)
    (distinct t ones)
   )
   (exists ((y (_ BitVec 4)))
    (and
     (mcb xlo xhi (bvadd (bvmul s y) t))
     (not (mulo s y))
     (not (addo (bvmul s y) t))
    )
   )
  )
 )
)

(define-fun LIT ((x (_ BitVec 4))) Bool
 (= (bvurem x s) t)
)

; valid xlo/xhi pairs
(assert (= (bvor (bvnot xlo) xhi) ones))

(assert
 (distinct
  (IC s t xlo xhi)
  (exists ((x (_ BitVec 4)))
   (and
    (mcb xlo xhi x)
    (LIT x)
   )
  )
 )
)

(check-sat)
(exit)

